package com.nokia.finance.tasks.service.common;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.PropertyNamingStrategies;
import com.fasterxml.jackson.databind.type.TypeFactory;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateTimeDeserializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateTimeSerializer;
import com.nokia.finance.tasks.dao.common.NginxLogDao;
import com.nokia.finance.tasks.pojo.po.common.NginxLogPo;
import com.nokia.finance.tasks.utils.AESUtil;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.LineBasedFrameDecoder;
import io.netty.handler.codec.string.StringDecoder;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.SmartLifecycle;
import org.springframework.stereotype.Service;

import java.net.InetSocketAddress;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;

/**
 * 接收nginx日志并记录到数据库
 */
@Slf4j
@Data
@Service
@ConfigurationProperties("nginx-log")
@RequiredArgsConstructor
public class NginxLogService implements SmartLifecycle {
    // 定义服务监听的端口号
    private Integer port;
    // 定义接收消息的最大长度
    private Integer maxLength;
    // 标志变量，用于标识服务是否正在运行
    private volatile boolean running = false;
    // Netty 的主事件循环组，用于接收客户端连接
    private EventLoopGroup parentGroup;
    // Netty 的子事件循环组，用于处理客户端连接后的数据读写
    private EventLoopGroup childGroup;
    // 保存绑定到指定端口的 ChannelFuture 对象
    private ChannelFuture channelFuture;
    // 注入的 Mapper 接口，用于操作数据库
    private final NginxLogDao nginxLogDao;

    /**
     * 启动方法，初始化并启动 TCP 服务
     */
    @Override
    public void start() {
        // 初始化主事件循环组和子事件循环组
        parentGroup = new NioEventLoopGroup();
        childGroup = new NioEventLoopGroup();
        try {
            // 使用 ServerBootstrap 配置并启动 Netty 服务端
            ServerBootstrap serverBootstrap = new ServerBootstrap()
                    .group(parentGroup, childGroup) // 设置主从线程组
                    .channel(NioServerSocketChannel.class) // 指定使用 NIO 通道
                    .childHandler(new ChannelInitializer<SocketChannel>() { // 配置子通道的处理器
                        @Override
                        protected void initChannel(SocketChannel socketChannel) {
                            // 添加 IP 过滤器，限制只允许本地连接
                            socketChannel.pipeline().addLast(new IpFilterHandler());
                            // 添加基于行的帧解码器，限制单行最大长度
                            socketChannel.pipeline().addLast(new LineBasedFrameDecoder(maxLength));
                            // 添加字符串解码器，将接收到的字节流转换为字符串
                            socketChannel.pipeline().addLast(new StringDecoder());
                            // 添加自定义的业务处理器，处理解析后的日志数据
                            socketChannel.pipeline().addLast(new SimpleChannelInboundHandler<String>() {
                                @Override
                                protected void channelRead0(ChannelHandlerContext channelHandlerContext, String s) throws Exception {
                                    // 创建 ObjectMapper 对象，用于 JSON 数据解析
                                    ObjectMapper mapper = new ObjectMapper();
                                    // 配置 ObjectMapper 忽略未知字段
                                    mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
                                    // 设置属性命名策略为蛇形命名法
                                    mapper.setPropertyNamingStrategy(PropertyNamingStrategies.SNAKE_CASE);
                                    // 创建 JavaTimeModule 模块，支持 LocalDateTime 的序列化和反序列化
                                    JavaTimeModule module = new JavaTimeModule();
                                    module.addSerializer(LocalDateTime.class,
                                            new LocalDateTimeSerializer(DateTimeFormatter.ISO_DATE_TIME));
                                    module.addDeserializer(LocalDateTime.class,
                                            new LocalDateTimeDeserializer(DateTimeFormatter.ISO_DATE_TIME));
                                    mapper.registerModule(module);
                                    // 将接收到的 JSON 字符串解析为 NginxLogPo 对象
                                    NginxLogPo po = mapper.readValue(s, NginxLogPo.class);
                                    // 获取日志中的 Token 字段
                                    String token = po.getToken();
                                    // 对 Token 进行 URL 解码
                                    String urlDecodeToken = URLDecoder.decode(token, StandardCharsets.UTF_8);
                                    // 对解码后的 Token 进行 AES 解密
                                    String decryptToken = AESUtil.decrypt(urlDecodeToken);
                                    // 将解密后的 Token 转换为 Map
                                    Map<String, String> map = mapper.readValue(decryptToken,
                                            TypeFactory.defaultInstance().constructMapType(Map.class, String.class,
                                                    String.class));
                                    // 从 Map 中解析时间戳，并格式化为 LocalDateTime
                                    LocalDateTime timeStamp = LocalDateTime.parse(map.get("TIME_STAMP"),
                                            DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                                    // 获取过期时间（秒）
                                    int expireTime = Integer.parseInt(map.get("EXPIRE_TIME"));
                                    // 计算过期时间戳
                                    LocalDateTime expireTimeStamp = timeStamp.plusSeconds(expireTime);
                                    // 设置 NginxLogPo 对象的其他字段
                                    po.setLoginId(map.get("LOGIN_ID"));
                                    po.setPageUrl(map.get("REQUEST_URL"));
                                    po.setAppId(map.get("APP_ID"));
                                    po.setToken(urlDecodeToken);
                                    po.setTimeStamp(timeStamp);
                                    po.setExpireTime(expireTime);
                                    po.setExpireTimeStamp(expireTimeStamp);
                                    // 将日志对象插入数据库
                                    nginxLogDao.insert(po);
                                }
                            });
                        }
                    })
                    .childOption(ChannelOption.SO_KEEPALIVE, true); // 设置保持连接选项
            // 绑定端口并同步等待绑定完成
            channelFuture = serverBootstrap.bind(port).sync();
            // 打印服务启动成功的日志
            log.info("TCP Server started on port: {}", port);
        } catch (InterruptedException e) {
            // 如果发生中断异常，设置线程中断状态并记录错误日志
            Thread.currentThread().interrupt();
            log.error(e.toString(), e);
            // 停止服务
            stop();
        }
    }

    /**
     * 停止方法，优雅地关闭服务
     */
    @Override
    public void stop() {
        // 设置运行标志为 false
        running = false;
        // 优雅地关闭主事件循环组
        if (parentGroup != null) {
            parentGroup.shutdownGracefully();
        }
        // 优雅地关闭子事件循环组
        if (childGroup != null) {
            childGroup.shutdownGracefully();
        }
        // 关闭绑定的 ChannelFuture
        if (channelFuture != null) {
            try {
                channelFuture.channel().closeFuture().sync();
            } catch (InterruptedException e) {
                // 如果发生中断异常，设置线程中断状态并记录错误日志
                Thread.currentThread().interrupt();
                log.error(e.toString(), e);
            }
        }
    }

    /**
     * 判断服务是否正在运行
     *
     * @return 如果服务正在运行则返回 true，否则返回 false
     */
    @Override
    public boolean isRunning() {
        return running;
    }

    /**
     * 内部类：IP 过滤处理器，限制只允许本地连接
     */
    private static class IpFilterHandler extends ChannelInboundHandlerAdapter {
        @Override
        public void channelActive(ChannelHandlerContext ctx) throws Exception {
            // 获取远程客户端的地址
            InetSocketAddress remoteAddress = (InetSocketAddress) ctx.channel().remoteAddress();
            String hostAddress = remoteAddress.getAddress().getHostAddress();
            // 如果远程地址不是本地地址，则拒绝连接
            if (!"127.0.0.1".equals(hostAddress)) {
                log.warn("Rejected connection from non-local IP: {}", hostAddress);
                ctx.close();
                return;
            }
            // 调用父类方法，继续处理连接
            super.channelActive(ctx);
        }
    }
}
